@@ -73,7 +73,7 @@
 #include <time.h>
 
 extern void gnuzip(int in, int out, char *origname, unsigned long timestamp, int level, int osflag, int rsync, int newrsync);
-extern void old_bzip2(int level, int suse_quirk);
+extern void old_bzip2(int level);
 
 #define BUFLEN		(64 * 1024)
 
@@ -132,6 +132,8 @@ static	void	gz_compress(int, int, const char *, uint32_t, int, int, int, int, in
 static	void	usage(void);
 static	void	display_version(void);
 static	void	display_license(void);
+static	void	shamble(char *, int, char *);
+static	void    rebrain(char *, char *, int, char *);
 
 int main(int, char **p);
 
@@ -151,6 +153,8 @@ static const struct option longopts[] = {
 	/* new options */
 	{ "gnu",                no_argument,            0,      'G' },
 	{ "old-bzip2",          no_argument,            0,      'O' },
+	{ "suse-bzip2",         no_argument,            0,      'S' },
+	{ "suse-pbzip2",        no_argument,            0,      'P' },
 	{ "zlib",               no_argument,            0,      'Z' },
 	{ "rsyncable",          no_argument,            0,      'R' },
 	{ "new-rsyncable",      no_argument,            0,      'r' },
@@ -173,6 +177,8 @@ main(int argc, char **argv)
 	const char *progname = argv[0];
 	int gnu = 0;
 	int bzold = 0;
+	int bzsuse = 0;
+	int pbzsuse = 0;
 	int quirks = 0;
 	char *origname = NULL;
 	uint32_t timestamp = 0;
@@ -181,7 +187,6 @@ main(int argc, char **argv)
 	int mflag = 0;
 	int fflag = 0;
 	int xflag = -1;
-	int suse_quirk = 0;
 	int ntfs_quirk = 0;
 	int level = 6;
 	int osflag = GZIP_OS_UNIX;
@@ -196,7 +201,7 @@ main(int argc, char **argv)
 		usage();
 	}
 
-#define OPT_LIST "123456789acdfhF:GLNnMmqRrT:Vo:k:s:ZO"
+#define OPT_LIST "123456789acdfhF:GLNnMmqRrT:Vo:k:s:ZOSP"
 
 	while ((ch = getopt_long(argc, argv, OPT_LIST, longopts, NULL)) != -1) {
 		switch (ch) {
@@ -206,6 +211,12 @@ main(int argc, char **argv)
 		case 'O':
 			bzold = 1;
 			break;
+		case 'S':
+			bzsuse = 1;
+			break;
+		case 'P':
+			pbzsuse = 1;
+			break;
 		case 'Z':
 			break;
 		case '1': case '2': case '3':
@@ -270,9 +281,6 @@ main(int argc, char **argv)
 				/* maximum compression but without indicating so */
 				level = 9;
 				xflag = 0;
-			} else if (strcmp(optarg, "suse") == 0) {
-				/* SuSe's patched bzip2. (Inexact emulation.) */
-				suse_quirk = 1;
 			} else {
 				fprintf(stderr, "%s: unknown quirk!\n", progname);
 				usage();
@@ -328,7 +336,15 @@ main(int argc, char **argv)
 		}
 		gnuzip(STDIN_FILENO, STDOUT_FILENO, origname, timestamp, level, osflag, rsync, new_rsync);
 	} else if (bzold) {
-		old_bzip2(level, suse_quirk);
+		if (quirks) {
+			fprintf(stderr, "%s: quirks not supported with --old-bzip2\n", progname);
+			return 1;
+		}
+		old_bzip2(level);
+	} else if (bzsuse) {
+		shamble("suse-bzip2/bzip2", level, "");
+	} else if (pbzsuse) {
+		rebrain("suse-bzip2", "pbzip2", level, "");
 	} else {
 		if (rsync || new_rsync) {
 			fprintf(stderr, "%s: --rsyncable not supported with --zlib\n", progname);
@@ -499,6 +515,25 @@ gz_compress(int in, int out, const char *origname, uint32_t mtime, int level, in
 	free(outbufp);
 }
 
+/* runs an external, reanimated compressor program */
+static	void
+shamble(char *zombie, int level, char *opts)
+{
+	char buf[128];
+	sprintf(buf, "%s/%s -%i %s", ZGZ_LIB, zombie, level, opts);
+	exit(system(buf));
+}
+
+/* swaps in a different library and runs a system program */
+static	void
+rebrain(char *zombie, char *program, int level, char *opts)
+{
+	char buf[128];
+	sprintf(buf, "LD_LIBRARY_PATH=%s/%s %s -%i %s",
+			ZGZ_LIB, zombie, program, level, opts);
+	exit(system(buf));
+}
+
 /* display usage */
 static void
 usage(void)
@@ -510,6 +545,8 @@ usage(void)
     " -G --gnu                 use GNU gzip implementation\n"
     " -Z --zlib                use zlib's implementation (default)\n"
     " -O --old-bzip2           generate bzip2 (0.9.5d) output\n"
+    " -S --suse-bzip2          generate suse bzip2 output\n"
+    " -P --suse-pbzip2         generate suse pbzip2 output\n"
     " -1 --fast                fastest (worst) compression\n"
     " -2 .. -8                 set compression level\n"
     " -9 --best                best (slowest) compression\n"
@@ -530,7 +567,7 @@ usage(void)
     " -R --rsyncable           make rsync-friendly archive\n"
     " -r --new-rsyncable       make rsync-friendly archive (new version)\n"
     " \nzlib-specific options:\n"
-    " -k --quirk QUIRK         enable a format quirk (buggy-bsd, ntfs, perl, suse)\n");
+    " -k --quirk QUIRK         enable a format quirk (buggy-bsd, ntfs, perl)\n");
 	exit(0);
 }
 
